/*
 *  soapui, copyright (C) 2005 Ole Matzura / eviware.com 
 *
 *  SoapUI is free software; you can redistribute it and/or modify it under the 
 *  terms of the GNU Lesser General Public License as published by the Free Software Foundation; 
 *  either version 2.1 of the License, or (at your option) any later version.
 *
 *  SoapUI is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
 *  even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 *  See the GNU Lesser General Public License for more details at gnu.org.
 */

package com.eviware.soapui;

import java.util.ArrayList;
import java.util.List;

import com.eviware.soapui.model.DesktopPanel;
import com.eviware.soapui.model.ModelItem;
import com.eviware.soapui.model.iface.Interface;
import com.eviware.soapui.model.iface.Operation;
import com.eviware.soapui.model.iface.Request;
import com.eviware.soapui.model.project.Project;
import com.eviware.soapui.model.support.InterfaceListenerAdapter;
import com.eviware.soapui.model.support.ProjectListenerAdapter;
import com.eviware.soapui.model.support.TestSuiteListenerAdapter;
import com.eviware.soapui.model.support.WorkspaceListenerAdapter;
import com.eviware.soapui.model.testsuite.TestCase;
import com.eviware.soapui.model.testsuite.TestStep;
import com.eviware.soapui.model.testsuite.TestSuite;
import com.eviware.soapui.model.workspace.Workspace;

public class SoapUIDesktop
{
	private final Workspace workspace;
	private final List<DesktopPanel> desktopPanels = new ArrayList<DesktopPanel>();
	private final InternalProjectListener projectListener = new InternalProjectListener();
	private final InternalInterfaceListener interfaceListener = new InternalInterfaceListener();
	private final InternalTestSuiteListener testSuiteListener = new InternalTestSuiteListener();

	public SoapUIDesktop( Workspace workspace )
	{
		this.workspace = workspace;
		initListeners();
	}
	
	private void initListeners()
	{
		workspace.addWorkspaceListener( new InternalWorkspaceListener() );
		
		for( int c = 0; c < workspace.getProjectCount(); c++ )
		{
			Project project = workspace.getProjectAt( c );
			project.addProjectListener( projectListener );
			
			for( int i = 0; i < project.getInterfaceCount(); i++ )
			{
				project.getInterfaceAt( i ).addInterfaceListener( interfaceListener );
			}

			for( int i = 0; i < project.getTestSuiteCount(); i++ )
			{
				project.getTestSuiteAt( i ).addTestSuiteListener( testSuiteListener );
			}
		}
	}

	public void closeDependantPanels( ModelItem modelItem )
	{
		DesktopPanel [] panels = SoapUI.getInstance().getDesktopPanels();
		
		for( int c = 0; c < panels.length; c++ )
		{
			if( panels[c].dependsOn( modelItem ))
			{
				closeDesktopPanel( panels[c] );
			}
		}
	}

	private void closeDesktopPanel(DesktopPanel panel)
	{
		SoapUI.getInstance().closeTab( panel.getModelItem() );
	}

	private class InternalWorkspaceListener extends WorkspaceListenerAdapter
	{
		public void projectRemoved(Project project)
		{
			project.removeProjectListener( projectListener );
			closeDependantPanels( project );
		}

		public void projectAdded(Project project)
		{
			project.addProjectListener( projectListener );
		}
	}
	
	private class InternalProjectListener extends ProjectListenerAdapter
	{
		public void interfaceRemoved(Interface iface)
		{
			iface.removeInterfaceListener( interfaceListener );
			closeDependantPanels( iface );
		}

		public void testSuiteRemoved(TestSuite testSuite)
		{
			testSuite.removeTestSuiteListener( testSuiteListener );
			closeDependantPanels( testSuite );
		}

		public void interfaceAdded(Interface iface)
		{
			iface.addInterfaceListener( interfaceListener );
		}

		public void testSuiteAdded(TestSuite testSuite)
		{
			testSuite.addTestSuiteListener( testSuiteListener );
		}
	}
	
	private class InternalInterfaceListener extends InterfaceListenerAdapter
	{
		public void operationRemoved(Operation operation)
		{
			closeDependantPanels( operation );
		}

		public void requestRemoved(Request request)
		{
			closeDependantPanels( request );
		}
	}

	private class InternalTestSuiteListener extends TestSuiteListenerAdapter
	{
		public void testCaseRemoved(TestCase testCase)
		{
			closeDependantPanels( testCase );
		}

		public void testStepRemoved(TestStep testStep, int index)
		{
			closeDependantPanels( testStep );
		}
	}
}
