/*
 *  soapui, copyright (C) 2005 Ole Matzura / eviware.com 
 *
 *  SoapUI is free software; you can redistribute it and/or modify it under the 
 *  terms of the GNU Lesser General Public License as published by the Free Software Foundation; 
 *  either version 2.1 of the License, or (at your option) any later version.
 *
 *  SoapUI is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
 *  even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 *  See the GNU Lesser General Public License for more details at gnu.org.
 */

package com.eviware.soapui.impl.wsdl.actions.request;

import java.awt.event.ActionEvent;
import java.util.ArrayList;
import java.util.List;

import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.JOptionPane;

import com.eviware.soapui.SoapUI;
import com.eviware.soapui.impl.wsdl.WsdlProject;
import com.eviware.soapui.impl.wsdl.WsdlRequest;
import com.eviware.soapui.impl.wsdl.WsdlTestCase;
import com.eviware.soapui.impl.wsdl.WsdlTestSuite;
import com.eviware.soapui.impl.wsdl.teststeps.WsdlTestRequestStep;
import com.eviware.soapui.model.testsuite.TestSuite;

/**
 * Adds a WsdlRequest to a WsdlTestCase
 * 
 * @author Ole.Matzura
 */

public class AddToTestCaseAction extends AbstractAction
{
   private final WsdlRequest request;

	public AddToTestCaseAction( WsdlRequest request )
   {
      super( "Add to Testcase" );
		this.request = request;
      putValue( Action.SHORT_DESCRIPTION, "Adds this request to a test-case" );
      putValue( Action.SMALL_ICON, SoapUI.createImageIcon("/addToTestCase.gif" ));
   }
   
   public void actionPerformed(ActionEvent e)
	{
      WsdlProject project = (WsdlProject) request.getOperation().getInterface().getProject();
      
      List<WsdlTestCase> testCases = new ArrayList<WsdlTestCase>();
      List<String> testCaseNames = new ArrayList<String>();
      WsdlTestCase testCase = null;
      
      if( project.getTestSuiteCount() == 0 )
      {
         String name = JOptionPane.showInputDialog( SoapUI.getInstance().getFrame(), "Missing Test-suite in project, enter name to create",
               "Create Test-Suite", JOptionPane.QUESTION_MESSAGE );
         if( name == null ) return;
         
         WsdlTestSuite testSuite = project.addNewTestSuite( name );
         SoapUI.getInstance().selectModelItem( testSuite );
         
         name = JOptionPane.showInputDialog( SoapUI.getInstance().getFrame(), "Enter name for test-case create",
               "Create Test-Case", JOptionPane.QUESTION_MESSAGE );
         if( name == null ) return;
        
         String stepName = JOptionPane.showInputDialog( SoapUI.getInstance().getFrame(), "Enter name for test step",
               request.getName(), JOptionPane.QUESTION_MESSAGE );
         if( stepName == null ) return;

         testCase = testSuite.addNewTestCase( name );
         WsdlTestRequestStep test = testCase.addTestRequestStep( request, stepName );
         
         SoapUI.getInstance().selectModelItem( test );
         SoapUI.getInstance().showDesktopPanel( test );
      }
      else
      {
         for( int c = 0; c < project.getTestSuiteCount(); c++ )
         {
            TestSuite testSuite = project.getTestSuiteAt( c );
            for( int i = 0; i < testSuite.getTestCaseCount(); i++ )
            {
               testCase = (WsdlTestCase) testSuite.getTestCaseAt( i );
               testCases.add( testCase );
               testCaseNames.add( (testCaseNames.size()+1) + ": " + testSuite.getName() + " - " + testCase.getName() );
            }
         }
         
         if( testCases.size() == 0 )
         {
         	List<String> testSuiteNames = new ArrayList<String>();
         	
            for( int c = 0; c < project.getTestSuiteCount(); c++ )
            {
               TestSuite testSuite = project.getTestSuiteAt( c );
               testSuiteNames.add( (testSuiteNames.size()+1) + ": " + testSuite.getName() );
            }
         	
            String selection = (String) JOptionPane.showInputDialog( SoapUI.getInstance().getFrame(), 
                  "Select test-suite", "Select test-suite to create test-case in", 
                  JOptionPane.QUESTION_MESSAGE, null, testSuiteNames.toArray(), null );
            if( selection == null ) return;
            
            WsdlTestSuite testSuite = (WsdlTestSuite) project.getTestSuiteAt( testSuiteNames.indexOf( selection ));
            
            String name = JOptionPane.showInputDialog( SoapUI.getInstance().getFrame(), "Enter name for test-case create",
                  "Create Test-Case", JOptionPane.QUESTION_MESSAGE );
            if( name == null ) return;
            
            testCase = testSuite.addNewTestCase( name );
         }
         else
         {
	         String selection = (String) JOptionPane.showInputDialog( SoapUI.getInstance().getFrame(), 
	               "Select test-case", "Select test-case", JOptionPane.QUESTION_MESSAGE, null, testCaseNames.toArray(), null );
	         if( selection == null ) return;
         
	         testCase = testCases.get( testCaseNames.indexOf( selection ));
         }

         String name = JOptionPane.showInputDialog( SoapUI.getInstance().getFrame(), "Specify name of test step", 
         		request.getName() );
         if( name == null ) return;
         
         WsdlTestRequestStep test = testCase.addTestRequestStep( request, name );
         
         SoapUI.getInstance().selectModelItem( test );
         SoapUI.getInstance().showDesktopPanel( test );
      }
   }
}
