/*
 *  soapui, copyright (C) 2005 Ole Matzura / eviware.com 
 *
 *  SoapUI is free software; you can redistribute it and/or modify it under the 
 *  terms of the GNU Lesser General Public License as published by the Free Software Foundation; 
 *  either version 2.1 of the License, or (at your option) any later version.
 *
 *  SoapUI is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
 *  even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 *  See the GNU Lesser General Public License for more details at gnu.org.
 */

package com.eviware.soapui.impl.wsdl.panels.iface;

import java.awt.Component;

import com.eviware.soapui.impl.EmptyPanelBuilder;
import com.eviware.soapui.impl.wsdl.WsdlInterface;
import com.eviware.soapui.model.DesktopPanel;
import com.eviware.soapui.support.PropertiesTable;

/**
 * PanelBuilder for WsdlInterface
 * 
 * @author Ole.Matzura
 */

public class WsdlInterfacePanelBuilder extends EmptyPanelBuilder
{
   private final WsdlInterface iface;

   public WsdlInterfacePanelBuilder(WsdlInterface iface)
   {
      this.iface = iface;
   }

   public Component buildOverviewPanel()
   {
   	PropertiesTable table = new PropertiesTable( "Interface Properties", iface );
   	table.addProperty( "Definition URL", "definition" );
   	table.addProperty( "Binding", "bindingName" );
   	table.addProperty( "PortType", "name" );
   	
   	return table;
   }

   public boolean hasOverviewPanel()
   {
      return true;
   }

   public DesktopPanel buildDesktopPanel()
   {
      return new WsdlInterfaceDesktopPanel( iface );
   }

   public boolean canBuildDesktopPanel()
   {
      return true;
   }

   public String getDesktopPanelTitle()
   {
      return iface.getProject().getName() + ":" + iface.getName();
   }
   
   
}
