/*
 *  soapui, copyright (C) 2005 Ole Matzura / eviware.com 
 *
 *  SoapUI is free software; you can redistribute it and/or modify it under the 
 *  terms of the GNU Lesser General Public License as published by the Free Software Foundation; 
 *  either version 2.1 of the License, or (at your option) any later version.
 *
 *  SoapUI is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
 *  even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 *  See the GNU Lesser General Public License for more details at gnu.org.
 */

package com.eviware.soapui.impl.wsdl.panels.request.support;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Set;

import javax.swing.ComboBoxModel;
import javax.swing.JOptionPane;
import javax.swing.event.ListDataEvent;
import javax.swing.event.ListDataListener;

import com.eviware.soapui.impl.wsdl.WsdlInterface;
import com.eviware.soapui.impl.wsdl.WsdlRequest;
import com.eviware.soapui.support.UISupport;

/**
 * ComboBox model for a request endpoint
 * 
 * @author Ole.Matzura
 */

public class EndpointsComboBoxModel implements ComboBoxModel, PropertyChangeListener
{
	private static final String ADD_NEW_ENDPOINT = "[add new endpoint..]";
	private Set<ListDataListener> listeners = new HashSet<ListDataListener>();
	private String[] endpoints;
	private WsdlRequest request;
	
	public EndpointsComboBoxModel( WsdlRequest request )
	{
		this.request = request;
		initEndpoints();
		request.addPropertyChangeListener( this );
		request.getOperation().getInterface().addPropertyChangeListener( this );
	}
	
	public void setSelectedItem(Object anItem)
	{
		if( anItem.equals( ADD_NEW_ENDPOINT ))
		{
			String value = (String) JOptionPane.showInputDialog( UISupport.getMainFrame(), 
					"Add new endpoint for interface [" + request.getOperation().getInterface().getName() + "]", 
					"Add new endpoint", JOptionPane.QUESTION_MESSAGE, null, null, request.getEndpoint() );
			if( value != null )
			{
				request.getOperation().getInterface().addEndpoint( value );
				request.setEndpoint( value );
			}
		}
		else
		{
			request.setEndpoint( (String)anItem );
		}
		
		notifyContentsChanged();
	}

	public void refresh()
	{
		initEndpoints();
		notifyContentsChanged();
	}

	private void initEndpoints()
	{
		endpoints = request.getOperation().getInterface().getEndpoints();
	}

	private void notifyContentsChanged()
	{
		Iterator<ListDataListener> iterator = listeners.iterator();
		ListDataEvent e = new ListDataEvent(this, ListDataEvent.CONTENTS_CHANGED, 0, getSize() );
		while( iterator.hasNext() )
		{
			iterator.next().contentsChanged( e );
		}
	}

	public Object getSelectedItem()
	{
		String endpoint = request.getEndpoint();
		return endpoint == null ? "- no endpoint set -" : endpoint;
	}

	public int getSize()
	{
		return endpoints.length + 1;
	}

	public Object getElementAt(int index)
	{
		return index == endpoints.length ? EndpointsComboBoxModel.ADD_NEW_ENDPOINT : endpoints[index];
	}

	public void addListDataListener(ListDataListener l)
	{
		listeners.add( l );
	}

	public void removeListDataListener(ListDataListener l)
	{
		listeners.remove( l );
	}

	public void propertyChange(PropertyChangeEvent evt)
	{
		String propertyName = evt.getPropertyName();
		
		if( propertyName.equals( WsdlRequest.ENDPOINT_PROPERTY ) )
      {
         notifyContentsChanged();
      }
		else if( propertyName.equals( WsdlInterface.ENDPOINT_PROPERTY ) )
      {
         refresh();
      }		
	}
	
	public void release()
	{
		request.removePropertyChangeListener( this );
		request.getOperation().getInterface().removePropertyChangeListener( this );
	}
}