/*
 *  soapui, copyright (C) 2005 Ole Matzura / eviware.com 
 *
 *  SoapUI is free software; you can redistribute it and/or modify it under the 
 *  terms of the GNU Lesser General Public License as published by the Free Software Foundation; 
 *  either version 2.1 of the License, or (at your option) any later version.
 *
 *  SoapUI is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
 *  even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 *  See the GNU Lesser General Public License for more details at gnu.org.
 */

package com.eviware.soapui.impl.wsdl.panels.testcase;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;

import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.BorderFactory;
import javax.swing.DefaultListModel;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;
import javax.swing.JTextArea;
import javax.swing.ListSelectionModel;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import com.eviware.soapui.SoapUI;
import com.eviware.soapui.impl.wsdl.teststeps.TransferResponseValuesTestStep;
import com.eviware.soapui.impl.wsdl.teststeps.WsdlTestRequestStep;
import com.eviware.soapui.impl.wsdl.teststeps.TransferResponseValuesTestStep.ValueTransfer;
import com.eviware.soapui.model.DesktopPanel;
import com.eviware.soapui.model.ModelItem;
import com.eviware.soapui.support.UISupport;
import com.eviware.soapui.support.XmlUtils;
import com.jgoodies.forms.builder.ButtonBarBuilder;

/**
 * DesktopPanel for TransferResponseValuesTestStep
 * 
 * @author Ole.Matzura
 */

public class TransferResponseValuesDesktopPanel extends JPanel implements DesktopPanel
{
	private final TransferResponseValuesTestStep transferStep;
	private DefaultListModel listModel;
	private JList transferList;
	private JTextArea sourceArea;
	private JTextArea targetArea;
	private JButton copyButton;
	private JButton deleteButton;
	private JButton declareButton;
	private JComboBox sourceTypeComboBox;
	private JComboBox targetTypeComboBox;

	public TransferResponseValuesDesktopPanel(TransferResponseValuesTestStep testStep)
	{
		super( new BorderLayout() );
		this.transferStep = testStep;
		buildUI();
	}

	private void buildUI()
	{
		JSplitPane splitPane = UISupport.createHorizontalSplit();
		
		listModel = new DefaultListModel();
		
		for( int c = 0; c < transferStep.getTransferCount(); c++ )
		{
			listModel.addElement( transferStep.getTransferAt( c  ).getName() );
		}
		
		transferList = new JList( listModel );
		transferList.setSelectionMode( ListSelectionModel.SINGLE_SELECTION );
		transferList.addListSelectionListener( new TransferListSelectionListener());
		
		JScrollPane listScrollPane = new JScrollPane( transferList );
		listScrollPane.setBorder( 
				BorderFactory.createCompoundBorder(
				BorderFactory.createTitledBorder( BorderFactory.createEmptyBorder(), "Value Transfers" ), 
				BorderFactory.createLineBorder( Color.GRAY )));
		
		splitPane.setLeftComponent( listScrollPane);
		
		JSplitPane innerSplit = UISupport.createVerticalSplit();
		sourceArea = new JTextArea();
		sourceArea.setEnabled( false );
		sourceArea.getDocument().addDocumentListener( new SourceAreaDocumentListener());
		
		targetArea = new JTextArea();
		targetArea.setEnabled( false );
		targetArea.getDocument().addDocumentListener( new TargetAreaDocumentListener());
		
		JPanel sourcePanel = new JPanel( new BorderLayout() );
		sourcePanel.add( new JScrollPane( sourceArea ), BorderLayout.CENTER );
		ButtonBarBuilder builder = new ButtonBarBuilder();
		builder.addFixed( new JLabel( "<html><b>Source XPath</b></html>"));
		builder.addGlue();
		builder.addFixed( new JLabel( "Select Source:"));
		builder.addRelatedGap();
		sourceTypeComboBox = new JComboBox( new Object[] { "RESPONSE", "REQUEST", "ENDPOINT", "DOMAIN", "USERNAME", "PASSWORD" });
		sourceTypeComboBox.addItemListener( new ItemListener(){

			public void itemStateChanged(ItemEvent e)
			{
				int ix = transferList.getSelectedIndex();
				if( ix != -1 )
				{
					String sourceType = sourceTypeComboBox.getSelectedItem().toString();
					transferStep.getTransferAt( ix ).setSourceType( sourceType );
					sourceArea.setEnabled( sourceType.equals( "REQUEST" ) || sourceType.equals( "RESPONSE" ) );
				}
			}} );
		builder.addFixed( sourceTypeComboBox);
		builder.setBorder( BorderFactory.createEmptyBorder( 3, 3, 3, 3 ));
		sourcePanel.add( builder.getPanel(), BorderLayout.NORTH );
		sourcePanel.setBorder( BorderFactory.createEmptyBorder( 3, 3, 3, 3 ));
		
		innerSplit.setTopComponent( sourcePanel );
		
		JPanel targetPanel = new JPanel( new BorderLayout() );
		targetPanel.add( new JScrollPane( targetArea ), BorderLayout.CENTER );
		builder = new ButtonBarBuilder();
		builder.addFixed( new JLabel( "<html><b>Target XPath</b></html>"));
		builder.addGlue();
		builder.addFixed( new JLabel( "Select Target:"));
		builder.addRelatedGap();
		targetTypeComboBox = new JComboBox( new Object[] { "REQUEST", "ENDPOINT", "DOMAIN", "USERNAME", "PASSWORD" });
		targetTypeComboBox.addItemListener( new ItemListener(){

			public void itemStateChanged(ItemEvent e)
			{
				int ix = transferList.getSelectedIndex();
				if( ix != -1 )
				{
					String targetType = targetTypeComboBox.getSelectedItem().toString();
					transferStep.getTransferAt( ix ).setTargetType( targetType );
					targetArea.setEnabled( targetType.equals( "REQUEST" ));
				}
			}} );
		builder.addFixed( targetTypeComboBox);
		builder.setBorder( BorderFactory.createEmptyBorder( 3, 3, 3, 3 ));
		targetPanel.add( builder.getPanel(), BorderLayout.NORTH );
		targetPanel.setBorder( BorderFactory.createEmptyBorder( 3, 3, 3, 3 ));
		
		innerSplit.setBottomComponent( targetPanel );
		
		innerSplit.setResizeWeight( 0.5 );
		innerSplit.setDividerLocation( 0.5 );
		
		splitPane.setRightComponent( innerSplit );
		splitPane.setResizeWeight( 0.1 );
		splitPane.setDividerLocation( 120 );
		
		add( splitPane, BorderLayout.CENTER );
		
		builder = new ButtonBarBuilder();
		builder.addFixed( new JButton( new AddAction() ) );
		builder.addRelatedGap();
		copyButton = new JButton( new CopyAction() );
		copyButton.setEnabled( false );
		builder.addFixed( copyButton);
		builder.addRelatedGap();
		deleteButton = new JButton( new DeleteAction() );
		deleteButton.setEnabled( false );
		builder.addFixed( deleteButton);
		builder.addRelatedGap();
		declareButton = new JButton( new DeclareNamespacesAction() );
		declareButton.setEnabled( false );
		builder.addFixed( declareButton);
		builder.addRelatedGap();
		builder.addFixed( new JButton( new RunAction() ));
		builder.addGlue();
		builder.addFixed( new JButton( new CloseAction() ));
		builder.setBorder( BorderFactory.createEmptyBorder( 3, 3, 3, 3 ) );
		
		add( builder.getPanel(), BorderLayout.SOUTH );
		
		setBorder( BorderFactory.createEmptyBorder( 3, 3, 3, 3 ));
		setPreferredSize( new Dimension( 550, 400 ));
		
		if( listModel.getSize() > 0 ) 
			transferList.setSelectedIndex( 0 );
	}
	
	private final class SourceAreaDocumentListener implements DocumentListener
	{
		public void insertUpdate(DocumentEvent e)
		{
			update();
		}

		private void update()
		{
			int ix = transferList.getSelectedIndex();
			if( ix != -1 )
			{
				transferStep.getTransferAt( ix ).setSourcePath( sourceArea.getText() );
			}
		}

		public void removeUpdate(DocumentEvent e)
		{
			update();
		}

		public void changedUpdate(DocumentEvent e)
		{
		}
	}
	
	private final class TargetAreaDocumentListener implements DocumentListener
	{
		public void insertUpdate(DocumentEvent e)
		{
			update();
		}

		private void update()
		{
			int ix = transferList.getSelectedIndex();
			if( ix != -1 )
			{
				transferStep.getTransferAt( ix ).setTargetPath( targetArea.getText() );
			}
		}

		public void removeUpdate(DocumentEvent e)
		{
			update();
		}

		public void changedUpdate(DocumentEvent e)
		{
		}
	}

	private final class TransferListSelectionListener implements ListSelectionListener
	{
		public void valueChanged(ListSelectionEvent e)
		{
			int ix = transferList.getSelectedIndex();
			if( ix == -1 )
			{
				sourceArea.setText( "" );
				targetArea.setText( "" );

				sourceArea.setEnabled( false );
				targetArea.setEnabled( false );
			}
			else
			{
				ValueTransfer config = transferStep.getTransferAt( ix );
				sourceArea.setText( config.getSourcePath() );
				targetArea.setText( config.getTargetPath() );
				
				String sourceType = config.getSourceType();
				String targetType = config.getTargetType();

				sourceTypeComboBox.setSelectedItem( sourceType );
				targetTypeComboBox.setSelectedItem( targetType );
				
				targetArea.setEnabled( targetType.equals( "REQUEST" ));
				sourceArea.setEnabled( sourceType.equals( "REQUEST" ) || sourceType.equals( "RESPONSE" ) );
			}
			
			copyButton.setEnabled( ix != -1 );
			deleteButton.setEnabled( ix != -1 );
			declareButton.setEnabled( ix != -1 );
			sourceTypeComboBox.setEnabled( ix != -1 );
			targetTypeComboBox.setEnabled( ix != -1 );
		}
	}

	private final class AddAction extends AbstractAction
	{
		public AddAction()
		{
			super( "Add" );
		}
		
		public void actionPerformed(ActionEvent e)
		{
			String name = UISupport.prompt( "Specify name for value transfer", "Add Transfer" );
			if( name == null || name.trim().length() == 0 ) return;
			
			transferStep.addTransfer( name );
			
			listModel.addElement( name );
			transferList.setSelectedIndex( listModel.getSize()-1 );
		}
	}
	
	private final class CopyAction extends AbstractAction
	{
		public CopyAction()
		{
			super( "Copy" );
		}
		
		public void actionPerformed(ActionEvent e)
		{
			int ix = transferList.getSelectedIndex();
			ValueTransfer config = transferStep.getTransferAt( ix );
			
			String name = UISupport.prompt( "Specify name for value transfer", "Copy Transfer", config.getName() );
			if( name == null || name.trim().length() == 0 ) return;
			
			ValueTransfer transfer = transferStep.addTransfer( name );
			transfer.setSourcePath( config.getSourcePath() );
			transfer.setTargetPath( config.getTargetPath() );
			transfer.setSourceType( config.getSourceType() );
			transfer.setTargetType( config.getTargetType() );
			
			listModel.addElement( name );
			transferList.setSelectedIndex( listModel.getSize()-1 );
		}
	}
	
	private final class DeleteAction extends AbstractAction
	{
		public DeleteAction()
		{
			super( "Delete" );
		}
		
		public void actionPerformed(ActionEvent e)
		{
			if( UISupport.confirm( "Delete selected transfer", "Delete Transfer" )) 
			{
				int ix = transferList.getSelectedIndex();
				transferStep.removeTransferAt( ix );
				listModel.remove( ix );
				
				if( listModel.getSize() > 0 )
					transferList.setSelectedIndex( ix > listModel.getSize()-1 ? listModel.getSize()-1 : ix );
			}
		}
	}
	
	private final class DeclareNamespacesAction extends AbstractAction
	{
		public DeclareNamespacesAction()
		{
			super( "Declare" );
			putValue( Action.SHORT_DESCRIPTION, "Declare available response/request namespaces in source/target expressions" );
		}
		
		public void actionPerformed(ActionEvent e)
		{
			try
			{
				WsdlTestRequestStep previousStep = (WsdlTestRequestStep) transferStep.getTestCase().findPreviousStepOfType( 
						transferStep, WsdlTestRequestStep.class );
				
				if (previousStep != null )
				{
					String xml = previousStep.getTestRequest().getResponseContent();
					if ( xml != null && xml.trim().length() > 0)
						sourceArea.setText(XmlUtils.declareXPathNamespaces(xml) + sourceArea.getText());
				}

				WsdlTestRequestStep nextStep = (WsdlTestRequestStep) transferStep.getTestCase().findNextStepOfType( 
						transferStep, WsdlTestRequestStep.class );
				
				if (nextStep != null )
				{
					String xml = nextStep.getTestRequest().getRequestContent();
					if ( xml != null && xml.trim().length() > 0)
						targetArea.setText(XmlUtils.declareXPathNamespaces(xml) + targetArea.getText());
				}
			}
			catch (Exception e1)
			{
				e1.printStackTrace();
			}
		}
	}
	
	private final class RunAction extends AbstractAction
	{
		public RunAction()
		{
			super( "Run" );
		}
		
		public void actionPerformed(ActionEvent e)
		{
			if( listModel.getSize() == 0 )
			{
				UISupport.showErrorMessage( "Missing transfers!" );
				return;
			}
			
			WsdlTestRequestStep previousStep = (WsdlTestRequestStep) transferStep.getTestCase().findPreviousStepOfType( 
					transferStep, WsdlTestRequestStep.class );
			
			if( previousStep == null )
			{
				UISupport.showErrorMessage("Missing previous request step" );
				return;
			}
			
			WsdlTestRequestStep nextStep = (WsdlTestRequestStep) transferStep.getTestCase().findNextStepOfType( 
					transferStep, WsdlTestRequestStep.class );
			
			if( nextStep == null )
			{
				UISupport.showErrorMessage("Missing next request step" );
				return;
			}
			
			transferStep.runTransfer( previousStep.getTestRequest(), nextStep.getTestRequest() );
		}
	}
	
	private final class CloseAction extends AbstractAction
	{
		public CloseAction()
		{
			super( "Close" );
		}
		
		public void actionPerformed(ActionEvent e)
		{
			SoapUI.getInstance().closeTab( transferStep );
		}
	}

	public ModelItem getModelItem()
	{
		return transferStep;
	}

	public boolean onClose()
	{
		return true;
	}

	public JComponent getComponent()
	{
		return this;
	}

	public boolean dependsOn(ModelItem modelItem)
	{
		return modelItem == transferStep || modelItem == transferStep.getTestCase() ||
				modelItem == transferStep.getTestCase().getTestSuite() ||
				modelItem == transferStep.getTestCase().getTestSuite().getProject();
	}
}
