/*
 *  soapui, copyright (C) 2005 Ole Matzura / eviware.com 
 *
 *  SoapUI is free software; you can redistribute it and/or modify it under the 
 *  terms of the GNU Lesser General Public License as published by the Free Software Foundation; 
 *  either version 2.1 of the License, or (at your option) any later version.
 *
 *  SoapUI is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
 *  even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 *  See the GNU Lesser General Public License for more details at gnu.org.
 */

package com.eviware.soapui.impl.wsdl.teststeps.actions;

import java.awt.event.ActionEvent;

import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.JOptionPane;

import com.eviware.soapui.SoapUI;
import com.eviware.soapui.impl.wsdl.WsdlTestCase;
import com.eviware.soapui.impl.wsdl.teststeps.WsdlTestRequest;
import com.eviware.soapui.impl.wsdl.teststeps.WsdlTestRequestStep;

/**
 * Clones a WsdlTestRequestStep
 * 
 * @author Ole.Matzura
 */

public class CloneTestRequestStepAction extends AbstractAction
{
   private final WsdlTestRequestStep testStep;

	public CloneTestRequestStepAction( WsdlTestRequestStep testStep )
   {
      super( "Clone" );
		this.testStep = testStep;
      putValue( Action.SHORT_DESCRIPTION, "Creates a copy of this request" );
      putValue( Action.SMALL_ICON, SoapUI.createImageIcon( "/clone_request.gif"));
   }
   
   public void actionPerformed(ActionEvent e)
	{
      WsdlTestRequest request = testStep.getTestRequest(); 
      
      String name = (String) JOptionPane.showInputDialog( SoapUI.getInstance().getFrame(), 
      		"Clone Test Request Step", "Specify name of cloned request step",
      		JOptionPane.QUESTION_MESSAGE, null, null, "Copy of " + request.getName() );
      if( name == null ) return;
      
      WsdlTestCase testCase = (WsdlTestCase) testStep.getTestCase();
      WsdlTestRequestStep newRequest = (WsdlTestRequestStep) testCase.cloneStep( testStep, name );
      
      SoapUI.getInstance().selectModelItem( newRequest );
      SoapUI.getInstance().showDesktopPanel( newRequest );
   }

}
