/*
 *  soapui, copyright (C) 2005 Ole Matzura / eviware.com 
 *
 *  SoapUI is free software; you can redistribute it and/or modify it under the 
 *  terms of the GNU Lesser General Public License as published by the Free Software Foundation; 
 *  either version 2.1 of the License, or (at your option) any later version.
 *
 *  SoapUI is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
 *  even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 *  See the GNU Lesser General Public License for more details at gnu.org.
 */

package com.eviware.soapui.impl.wsdl.teststeps.assertions;

import org.apache.xmlbeans.XmlError;

/**
 * Holder for an assertion error
 * 
 * @author Ole.Matzura
 */

public class AssertionError
{
	private String message;
	private XmlError xmlError;
	
	public AssertionError( String message )
	{
		this.message = message;
	}
	
	public AssertionError( XmlError xmlError )
	{
		this.xmlError = xmlError;
		this.message = xmlError.getMessage();
	}
	
	public String getMessage()
	{
		return message;
	}
	
	public int getLineNumber()
	{
		return xmlError == null ? -1 : xmlError.getLine();
	}
	
	public XmlError getXmlError()
	{
		return xmlError;
	}

	public String toString()
	{
		if( xmlError == null ) 
			return message;
		
		return "line " + getLineNumber() + ": " + message;
	}

	public boolean equals(Object obj)
	{
		if( obj == null ) 
			return false;
		
		if( obj.getClass().getName().equals( this.getClass().getName() ) &&
		 	 obj.getClass().getClassLoader().equals( this.getClass().getClassLoader() ))
		{
			AssertionError errorObj = (AssertionError) obj;
			return errorObj.toString().equals( toString() );
		}
		
		return super.equals(obj);
	}
}
