/*
 *  soapui, copyright (C) 2005 Ole Matzura / eviware.com 
 *
 *  SoapUI is free software; you can redistribute it and/or modify it under the 
 *  terms of the GNU Lesser General Public License as published by the Free Software Foundation; 
 *  either version 2.1 of the License, or (at your option) any later version.
 *
 *  SoapUI is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
 *  even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 *  See the GNU Lesser General Public License for more details at gnu.org.
 */

package com.eviware.soapui.model.tree.nodes;

import com.eviware.soapui.model.iface.Interface;
import com.eviware.soapui.model.iface.InterfaceListener;
import com.eviware.soapui.model.iface.Operation;
import com.eviware.soapui.model.iface.Request;
import com.eviware.soapui.model.tree.AbstractTreeNode;
import com.eviware.soapui.model.tree.SoapUITreeNode;

/**
 * SoapUITreeNode for Interface implementations
 * 
 * @author Ole.Matzura
 */

public class InterfaceTreeNode extends AbstractTreeNode
{
   private final Interface iface;
	private InternalInterfaceListener internalInterfaceListener;

   public InterfaceTreeNode( Interface iface )
   {
      super( iface, iface.getProject() );
      this.iface = iface;
      
      internalInterfaceListener = new InternalInterfaceListener();
		iface.addInterfaceListener( internalInterfaceListener);
   }
  
   public void release()
   {
   	super.release();
   	
   	iface.removeInterfaceListener( internalInterfaceListener );
   }
   
   public int getChildCount()
   {
      return iface.getOperationCount();
   }

   public int getIndexOfChild(Object child)
   {
      for( int c = 0; c < iface.getOperationCount(); c++ )
      {
         if( iface.getOperationAt( c ).getTreeNode() == child ) return c;
      }
      
      return -1;
   }

   public SoapUITreeNode getChildNode(int index)
   {
      return iface.getOperationAt(index).getTreeNode();
   }
   
   private class InternalInterfaceListener extends InternalTreeNodeListener implements InterfaceListener
   {
   	public InternalInterfaceListener()
   	{
   		super( getTreeModel() );
   	}

      public void requestAdded(Request request)
      {
         notifyNodeInserted( request );
      }

      public void requestRemoved(Request request)
      {
         notifyNodeRemoved( request );
      }

      public void operationAdded(Operation operation)
      {
         notifyNodeInserted( operation );
      }
         
      public void operationRemoved(Operation operation)
      {
         notifyNodeRemoved( operation );
      }
   }
}
