/*
 *  soapui, copyright (C) 2005 Ole Matzura / eviware.com 
 *
 *  SoapUI is free software; you can redistribute it and/or modify it under the 
 *  terms of the GNU Lesser General Public License as published by the Free Software Foundation; 
 *  either version 2.1 of the License, or (at your option) any later version.
 *
 *  SoapUI is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
 *  even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 *  See the GNU Lesser General Public License for more details at gnu.org.
 */

package com.eviware.soapui.model.tree.nodes;

import com.eviware.soapui.SoapUI;
import com.eviware.soapui.model.iface.Interface;
import com.eviware.soapui.model.project.Project;
import com.eviware.soapui.model.project.ProjectListener;
import com.eviware.soapui.model.testsuite.TestSuite;
import com.eviware.soapui.model.tree.AbstractTreeNode;
import com.eviware.soapui.model.tree.SoapUITreeNode;

/**
 * SoapUITreeNode for Project implementations
 * 
 * @author Ole.Matzura
 */

public class ProjectTreeNode extends AbstractTreeNode
{
	private InternalProjectListener internalProjectListener;

   public ProjectTreeNode(Project project )
   {
      super( project, SoapUI.getInstance().getWorkspace() );
      
      internalProjectListener = new InternalProjectListener();
		project.addProjectListener( internalProjectListener);
   }
   
   public void release()
   {
   	super.release();
   	
   	getProject().removeProjectListener( internalProjectListener );
   }

   public int getChildCount()
   {
     return getProject().getInterfaceCount() + getProject().getTestSuiteCount();
   }

   public int getIndexOfChild(Object child)
   {
      Project project = getProject();
		for( int c = 0; c < project.getInterfaceCount(); c++ )
      {
         if( project.getInterfaceAt( c ).getTreeNode() == child ) return c;
      }
      
      for( int c = 0; c < project.getTestSuiteCount(); c++ )
      {
         if( project.getTestSuiteAt( c ).getTreeNode() == child ) 
         	return c + project.getInterfaceCount();
      }
      
      return -1;
   }

   public SoapUITreeNode getChildNode(int index)
   {
      Project project = getProject();
		if( index < project.getInterfaceCount() )
         return project.getInterfaceAt( index ).getTreeNode();
      else
         return project.getTestSuiteAt( index-project.getInterfaceCount() ).getTreeNode();
   }

   public Project getProject()
   {
      return (Project) getModelItem();
   }
   
   private class InternalProjectListener extends InternalTreeNodeListener implements ProjectListener
   {
   	public InternalProjectListener()
   	{
   		super( getTreeModel() );
   	}

      public void interfaceAdded(Interface iface)
      {
         notifyNodeInserted( iface );
      }
         
      public void interfaceRemoved(Interface iface)
      {
         notifyNodeRemoved( iface );
      }
      public void testSuiteAdded(TestSuite testSuite)
      {
         notifyNodeInserted( testSuite );
      }

      public void testSuiteRemoved(TestSuite testSuite)
      {
         notifyNodeRemoved( testSuite );
      }
   }
   
}
