/*
 *  soapui, copyright (C) 2005 Ole Matzura / eviware.com 
 *
 *  SoapUI is free software; you can redistribute it and/or modify it under the 
 *  terms of the GNU Lesser General Public License as published by the Free Software Foundation; 
 *  either version 2.1 of the License, or (at your option) any later version.
 *
 *  SoapUI is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
 *  even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 *  See the GNU Lesser General Public License for more details at gnu.org.
 */

package com.eviware.soapui.support.actions;

import java.awt.event.ActionEvent;
import java.awt.event.KeyEvent;
import java.io.StringWriter;

import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.KeyStroke;

import org.apache.log4j.Logger;

import com.eviware.soapui.SoapUI;
import com.eviware.soapui.support.JXmlTextArea;
import com.eviware.soapui.support.XmlUtils;

/**
 * Formats the XML of a JXmlTextArea
 * 
 * @author Ole.Matzura
 */

public class FormatXmlAction extends AbstractAction
{
	private final static Logger log = Logger.getLogger( FormatXmlAction.class );
   private final JXmlTextArea textArea;

	public FormatXmlAction( JXmlTextArea textArea)
   {
      super( "Format XML" );
      putValue( Action.SMALL_ICON, SoapUI.createImageIcon( "/format_request.gif"));
      putValue( Action.SHORT_DESCRIPTION, "Pretty-prints the request xml" );
      putValue( Action.ACCELERATOR_KEY, KeyStroke.getKeyStroke( KeyEvent.VK_F, KeyEvent.ALT_MASK ));
		this.textArea = textArea;
   }
   
	public void actionPerformed(ActionEvent e)
	{
     try
      {
         org.w3c.dom.Document dom = XmlUtils.parseXml(textArea.getText());
         StringWriter writer = new StringWriter();
         XmlUtils.serializePretty(dom, writer);
         textArea.setText(writer.toString()); 
         textArea.setCaretPosition( 0 );
      }
      catch (Exception e1)
      {
         log.error( e1.getMessage() );
      }  
   }
}
