/*
 *  soapui, copyright (C) 2005 Ole Matzura / eviware.com 
 *
 *  SoapUI is free software; you can redistribute it and/or modify it under the 
 *  terms of the GNU Lesser General Public License as published by the Free Software Foundation; 
 *  either version 2.1 of the License, or (at your option) any later version.
 *
 *  SoapUI is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
 *  even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 *  See the GNU Lesser General Public License for more details at gnu.org.
 */

package com.eviware.soapui.model.impl.wsdl.teststeps;

import javax.swing.ImageIcon;

import junit.framework.TestCase;

import com.eviware.soapui.config.CallConfig;
import com.eviware.soapui.config.TestStepConfig;
import com.eviware.soapui.config.TransferValuesStepConfig;
import com.eviware.soapui.config.ValueTransferConfig;
import com.eviware.soapui.impl.wsdl.teststeps.TransferResponseValuesTestStep;
import com.eviware.soapui.impl.wsdl.teststeps.WsdlTestRequest;
import com.eviware.soapui.model.iface.Interface;
import com.eviware.soapui.model.iface.InterfaceListener;
import com.eviware.soapui.model.iface.Operation;
import com.eviware.soapui.model.iface.Request;
import com.eviware.soapui.model.iface.RequestBuilder;
import com.eviware.soapui.model.project.Project;
import com.eviware.soapui.model.support.AbstractModelItem;
import com.eviware.soapui.model.tree.SoapUITreeNode;

/**
 * Tests various types of transfers..
 * 
 * @author Ole.Matzura
 */

public class TransferResponseValuesTestCase extends TestCase
{
	private TransferResponseValuesTestStep testStep;
	private TransferValuesStepConfig config;
	private WsdlTestRequest sourceRequest;
	private WsdlTestRequest targetRequest;

	protected void setUp() throws Exception
	{
		config = TransferValuesStepConfig.Factory.newInstance();
		ValueTransferConfig transfer = config.addNewTransfers();
		transfer.setSourceType( "ENDPOINT" );
		transfer.setTargetType( "ENDPOINT" );
		
		transfer = config.addNewTransfers();
		transfer.setSourceType( "DOMAIN" );
		transfer.setTargetType( "DOMAIN" );
		
		transfer = config.addNewTransfers();
		transfer.setSourceType( "REQUEST" );
		transfer.setSourcePath( "//test" );
		transfer.setTargetType( "USERNAME" );
		
		transfer = config.addNewTransfers();
		transfer.setSourceType( "RESPONSE" );
		transfer.setSourcePath( "//test" );
		transfer.setTargetType( "PASSWORD" );
		
		transfer = config.addNewTransfers();
		transfer.setSourceType( "ENDPOINT" );
		transfer.setTargetType( "REQUEST" );
		transfer.setTargetPath( "//test" );
		
		transfer = config.addNewTransfers();
		transfer.setSourceType( "RESPONSE" );
		transfer.setSourcePath( "//test2" );
		transfer.setTargetType( "REQUEST" );
		transfer.setTargetPath( "//test2" );
		
		TestStepConfig testStepConfig = TestStepConfig.Factory.newInstance();
		testStepConfig.setConfig( config  );
		
		testStep = new TransferResponseValuesTestStep( null, testStepConfig );
		
		CallConfig sourceCallConfig = CallConfig.Factory.newInstance();
		CallConfig targetCallConfig = CallConfig.Factory.newInstance();
		
		sourceRequest = new WsdlTestRequest( new MockOperation(), sourceCallConfig, null );
		sourceRequest.setRequest( "<bil><test>hello</test></bil>" );
		sourceRequest.setResponse( "<bil><test>hello2</test><test2>hellothere</test2></bil>" );
		
		targetRequest = new WsdlTestRequest( new MockOperation(), targetCallConfig, null );
		targetRequest.setRequest( "<bil><test>hello</test><test2>bu</test2></bil>" );
	}
	
	public void testTransfer() throws Exception
	{
		sourceRequest.setEndpoint( "BIL" );
		sourceRequest.setDomain( "mydomain" );
		
		testStep.runTransfer( sourceRequest, targetRequest );
		
		assertEquals( "BIL", targetRequest.getEndpoint() );
		assertEquals( "mydomain", targetRequest.getDomain() );
		assertEquals( "hello", targetRequest.getUsername() );
		assertEquals( "hello2", targetRequest.getPassword() );
		assertEquals( "<bil><test>BIL</test><test2>hellothere</test2></bil>", targetRequest.getRequestContent() );
	}
	
	private class MockOperation extends AbstractModelItem implements Operation
	{
		private MockInterface mockInterface;

		private MockOperation()
		{
			mockInterface = new MockInterface();
		}

		protected SoapUITreeNode createTreeNode()
		{
			return null;
		}

		public String getAction()
		{
			return null;
		}

		public Request getRequestAt(int index)
		{
			return null;
		}

		public int getRequestCount()
		{
			return 0;
		}

		public Interface getInterface()
		{
			return mockInterface;
		}

		public String getName()
		{
			return null;
		}

		public ImageIcon getIcon()
		{
			return null;
		}}
	
	   private class MockInterface extends AbstractModelItem implements Interface
	   {
			protected SoapUITreeNode createTreeNode()
			{
				return null;
			}

			public String[] getEndpoints()
			{
				return new String[] {"mockendpoint"};
			}

			public void addEndpoint(String endpoint)
			{
			}

			public void removeEndpoint(String endpoint)
			{
			}

			public void changeEndpoint(String oldEndpoint, String newEndpoint)
			{
			}

			public Operation getOperationAt(int index)
			{
				return null;
			}

			public int getOperationCount()
			{
				return 0;
			}

			public Project getProject()
			{
				return null;
			}

			public String getDefinition()
			{
				return null;
			}

			public void addInterfaceListener(InterfaceListener listener)
			{
			}

			public void removeInterfaceListener(InterfaceListener listener)
			{
			}

			public RequestBuilder getRequestBuilder()
			{
				return null;
			}

			public String getName()
			{
				return null;
			}

			public ImageIcon getIcon()
			{
				return null;
			}

			public Operation getOperationByName(String name)
			{
				// TODO Auto-generated method stub
				return null;
			}}
}
