/*
 *  soapui, copyright (C) 2005 Ole Matzura / eviware.com 
 *
 *  SoapUI is free software; you can redistribute it and/or modify it under the 
 *  terms of the GNU Lesser General Public License as published by the Free Software Foundation; 
 *  either version 2.1 of the License, or (at your option) any later version.
 *
 *  SoapUI is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
 *  even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 *  See the GNU Lesser General Public License for more details at gnu.org.
 */

package com.eviware.soapui.support;

import junit.framework.TestCase;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;

public class XmlUtilsTestCase extends TestCase
{
	public void testGetElementIndex() throws Exception
	{
		Document dom = XmlUtils.parseXml( "<h1><p>p1</p><h2>lkj</h2><p>p2</p></h1>");
		NodeList nl = dom.getDocumentElement().getElementsByTagName( "p" );
		
		assertEquals( 1, XmlUtils.getElementIndex( (Element) nl.item(0)));
		assertEquals( 2, XmlUtils.getElementIndex( (Element) nl.item(1)));
	}
	
	public void testGetElementPath() throws Exception
	{
		Document dom = XmlUtils.parseXml( "<h1><p>p1</p><h2>lkj</h2><p>p2</p></h1>");
		NodeList nl = dom.getDocumentElement().getElementsByTagName( "p" );
		
		assertEquals( "/h1[1]/p[1]", XmlUtils.getElementPath( (Element) nl.item(0)) );
		assertEquals( "/h1[1]/p[2]", XmlUtils.getElementPath( (Element) nl.item(1)) );
	}
	
	public void testTransferValues() throws Exception
	{
		String doc1 = "<h1><p>p1</p><h2 test=\"bil\">lkj</h2></h1>";
		String doc2 = "<h1><p>string</p><h2>string</h2><p>p2</p></h1>";
		
		String result = XmlUtils.transferValues( doc1, doc2 );
		assertEquals( "<h1><p>p1</p><h2 test=\"bil\">lkj</h2><p>p2</p></h1>", result );
	}
	
	public void testTransferValuesNS() throws Exception
	{
		String doc1 = "<ns:h1 xmlns:ns=\"test\"><ns:p>p1</ns:p><ns:h2 test=\"bil\">lkj</ns:h2></ns:h1>";
		String doc2 = "<ns:h1 xmlns:ns=\"test\"><ns:p>string</ns:p><ns:h2>string</ns:h2><ns:p>p2</ns:p></ns:h1>";
		
		String result = XmlUtils.transferValues( doc1, doc2 );
		assertEquals( "<ns:h1 xmlns:ns=\"test\"><ns:p>p1</ns:p><ns:h2 test=\"bil\">lkj</ns:h2><ns:p>p2</ns:p></ns:h1>", result );
	}
}
